/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('user-profile-entity', function () {

  juju.components.UserProfileEntity = React.createClass({

    propTypes: {
      changeState: React.PropTypes.func,
      entity: React.PropTypes.object.isRequired,
      expanded: React.PropTypes.bool,
      getDiagramURL: React.PropTypes.func,
      switchModel: React.PropTypes.func,
      type: React.PropTypes.string.isRequired
    },

    /**
      Generate the initial state for the component.
       @method getInitialState
    */
    getInitialState: function getInitialState() {
      return {
        expanded: false,
        styles: {
          height: '0px',
          opacity: 0
        }
      };
    },

    /**
      Called once the component has initially mounted.
       @method componentDidMount
    */
    componentDidMount: function componentDidMount() {
      // If the component should initially be shown as expanded then animate it
      // open.
      if (this.props.expanded) {
        this._toggle();
      }
    },

    /**
      Generate the base class names for the component.
       @method _generateClasses
      @returns {Object} The collection of class names.
    */
    _generateClasses: function _generateClasses() {
      return classNames('user-profile__entity', 'user-profile__list-row', 'twelve-col', {
        'user-profile__entity--expanded': this.state.expanded
      });
    },

    /**
      Calls to switch the env to the one the user clicked on.
       @method _switchEnv
      @param {String} uuid The model uuid.
      @param {String} name The model name.
      @param {Object} e The click event.
    */
    _switchEnv: function _switchEnv(uuid, name, e) {
      this.props.switchModel(uuid, name);
    },

    /**
      Navigate to the entity details.
       @method _viewEntity
      @param {Object} e The click event.
      @param {String} id The entity id.
    */
    _viewEntity: function _viewEntity(id, e) {
      this.props.changeState({
        sectionC: {
          component: 'charmbrowser',
          metadata: {
            activeComponent: 'entity-details',
            id: id.replace('cs:', '')
          }
        }
      });
    },

    /**
      Toggle between the summary and details.
       @method _toggle
    */
    _toggle: function _toggle() {
      var _this = this;

      var expanded = this.state.expanded;
      this.setState({ expanded: !expanded }, function () {
        var newHeight = expanded ? '0px' : _this.refs.inner.offsetHeight + 'px';
        _this.setState({ styles: {
            height: newHeight,
            opacity: expanded ? 0 : 1
          } });
      });
    },

    /**
      Handle clicks on tags.
       @method _handleTagClick
      @param {Object} e The event.
      @param {String} tag The tag to view.
    */
    _handleTagClick: function _handleTagClick(tag, e) {
      this.props.changeState({
        sectionC: {
          component: 'charmbrowser',
          metadata: {
            activeComponent: 'search-results',
            search: null,
            tags: tag
          }
        }
      });
    },

    /**
      Generate a list of tags.
       @method _generateTags
      @returns {Object} A list of tag components.
    */
    _generateTags: function _generateTags() {
      var _this2 = this;

      var tags = [];
      var entity = this.props.entity;
      var tagList = entity.tags;
      if (!tagList) {
        return;
      }
      tagList.forEach(function (tag) {
        tags.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item link',
            key: entity.id + '-' + tag,
            onClick: _this2._handleTagClick.bind(_this2, tag),
            role: 'button',
            tabIndex: '0' },
          tag
        ));
      });
      return React.createElement(
        'div',
        { className: 'twelve-col no-margin-bottom' },
        React.createElement(
          'div',
          { className: 'two-col' },
          'Tags'
        ),
        React.createElement(
          'ul',
          { className: 'ten-col last-col' },
          tags
        )
      );
    },

    /**
      Generate a list of commits.
       @method _generateTags
      @returns {Object} A list of commits.
    */
    _generateCommits: function _generateCommits() {
      var commits = [];
      var entity = this.props.entity;
      var commitList = entity.commits;
      if (!commitList) {
        return;
      }
      commitList.forEach(function (commit) {
        commits.push(React.createElement(
          'li',
          { key: entity.id + '-' + commit.id },
          React.createElement(
            'div',
            { className: 'seven-col' },
            commit.description
          ),
          React.createElement(
            'div',
            { className: 'three-col' },
            commit.time
          ),
          React.createElement(
            'div',
            { className: 'two-col last-col' },
            commit.author
          )
        ));
      });
      return React.createElement(
        'div',
        { className: 'user-profile__entity-commits twelve-col no-margin-bottom' },
        React.createElement(
          'div',
          { className: 'twelve-col' },
          'Latest commits',
          React.createElement(
            'span',
            { className: 'user-profile__size' },
            '(',
            commitList.length,
            ' commits)'
          )
        ),
        React.createElement(
          'ul',
          { className: 'user-profile__entity-commit-list twelve-col no-margin-bottom' },
          commits
        )
      );
    },

    /**
      Generate the diagram for a bundle.
       @method _generateDiagram
      @return {Object} The diagram.
    */
    _generateDiagram: function _generateDiagram() {
      if (this.props.type !== 'bundle') {
        return;
      }
      var url = this.props.getDiagramURL(this.props.entity.id);
      return React.createElement(
        'div',
        { className: 'user-profile__entity-diagram twelve-col' },
        React.createElement('object', { type: 'image/svg+xml', data: url,
          className: 'entity-content__diagram-image' })
      );
    },

    /**
      Generate the description.
       @method _generateDescription
      @return {Object} The description.
    */
    _generateDescription: function _generateDescription() {
      var description = this.props.entity.description;
      if (!description) {
        return;
      }
      return React.createElement(
        'div',
        { className: 'twelve-col no-margin-bottom' },
        React.createElement(
          'div',
          { className: 'two-col' },
          'Description'
        ),
        React.createElement(
          'div',
          { className: 'ten-col last-col' },
          description
        )
      );
    },

    /**
      Generate the series for a charm.
       @method _generateSeries
      @return {Object} The series component.
    */
    _generateSeries: function _generateSeries() {
      if (this.props.type !== 'charm') {
        return;
      }
      return React.createElement(
        'div',
        { className: 'nine-col' },
        'Series: ',
        this.props.entity.series
      );
    },

    /**
      Generate the services for a bundle.
       @method _generateServices
      @return {Object} The services component.
    */
    _generateServices: function _generateServices() {
      if (this.props.type !== 'bundle') {
        return;
      }
      var entity = this.props.entity;
      var services = [];
      var names = Object.keys(entity.services);
      names.forEach(function (name) {
        services.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item',
            key: entity.id + '-service-' + name },
          name
        ));
      });
      return React.createElement(
        'div',
        { className: 'nine-col' },
        'Composed of:',
        React.createElement(
          'ul',
          { className: 'user-profile__entity-service-list' },
          services
        )
      );
    },

    render: function render() {
      var props = this.props;
      var entity = props.entity;
      var type = props.type;
      var isModel = type === 'model';
      var isCharm = type === 'charm';
      var name = entity.name;
      var id = isModel ? entity.uuid : entity.id;
      var buttonAction = isModel ? this._switchEnv.bind(this, id, name) : this._viewEntity.bind(this, id);
      var icon = isCharm ? React.createElement('img', { className: 'user-profile__entity-icon',
        src: entity.icon,
        title: name }) : undefined;
      return React.createElement(
        'li',
        { className: this._generateClasses(),
          onClick: this._toggle },
        React.createElement(
          'div',
          { className: 'user-profile__entity-summary twelve-col no-margin-bottom' },
          this.props.children
        ),
        React.createElement(
          'div',
          { className: 'user-profile__entity-details twelve-col',
            style: this.state.styles },
          React.createElement(
            'div',
            { className: 'twelve-col no-margin-bottom',
              ref: 'inner' },
            React.createElement(
              'div',
              { className: 'user-profile__entity-details-header twelve-col' },
              React.createElement(
                'div',
                { className: 'ten-col no-margin-bottom' },
                icon,
                name
              ),
              React.createElement(
                'div',
                { className: 'user-profile__entity-details-header-action ' + 'two-col last-col no-margin-bottom' },
                React.createElement(juju.components.GenericButton, {
                  action: buttonAction,
                  title: isModel ? 'Manage' : 'View' })
              )
            ),
            React.createElement(
              'div',
              { className: 'user-profile__entity-details-content ' + 'twelve-col no-margin-bottom' },
              this._generateSeries(),
              this._generateServices(),
              React.createElement(
                'div',
                { className: 'three-col last-col' },
                'Owner: ',
                entity.owner
              ),
              this._generateDiagram(),
              this._generateDescription(),
              this._generateTags(),
              this._generateCommits()
            )
          )
        )
      );
    }
  });
}, '', {
  requires: ['generic-button']
});