/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('UserProfile', function () {
  var models, charmstore, charms, bundles, users, env;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('user-profile', function () {
      done();
    });
  });

  beforeEach(function () {
    models = [{
      uuid: 'env1',
      name: 'sandbox',
      lastConnection: 'today',
      owner: 'test-owner'
    }];
    var list = sinon.stub();
    var charm = jsTestUtils.makeEntity().toEntity();
    charm.series = [charm.series];
    charms = [charm];
    var bundle = jsTestUtils.makeEntity(true).toEntity();
    bundle.series = [bundle.series];
    bundles = [bundle];
    list.withArgs('test-owner', sinon.match.any, 'charm').callsArgWith(1, null, charms);
    list.withArgs('test-owner', sinon.match.any, 'bundle').callsArgWith(1, null, bundles);
    charmstore = {
      list: list,
      version: '9',
      url: 'example.com/'
    };
    users = { charmstore: {
        user: 'test-owner',
        usernameDisplay: 'test owner'
      } };
    env = {
      findFacadeVersion: sinon.stub(),
      get: sinon.stub().returns('default')
    };
  });

  it('renders the empty state', function () {
    var jem = {
      listEnvironments: sinon.stub().callsArgWith(0, null, { envs: [] })
    };
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      charmstore: {},
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      jem: jem,
      switchModel: sinon.stub(),
      showConnectingMask: sinon.stub(),
      interactiveLogin: true,
      changeState: sinon.stub(),
      storeUser: sinon.stub(),
      user: users.charmstore }), true);
    var instance = component.getMountedInstance();
    var output = component.getRenderOutput();
    var expected = React.createElement(
      juju.components.Panel,
      {
        instanceName: 'user-profile',
        visible: true },
      React.createElement(
        'div',
        { className: 'twelve-col' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(juju.components.UserProfileHeader, {
            users: users,
            avatar: '',
            bundleCount: 0,
            charmCount: 0,
            environmentCount: 0,
            interactiveLogin: instance._interactiveLogin,
            username: users.charmstore.usernameDisplay }),
          React.createElement(
            'div',
            { className: 'user-profile__empty twelve-col no-margin-bottom' },
            React.createElement('img', { alt: 'Empty profile',
              className: 'user-profile__empty-image',
              src: '/juju-ui/assets/images/non-sprites/empty_profile.png' }),
            React.createElement(
              'h2',
              { className: 'user-profile__empty-title' },
              'Your profile is currently empty'
            ),
            React.createElement(
              'p',
              { className: 'user-profile__empty-text' },
              'Your models, bundles and charms will appear here when you create them.'
            )
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('displays loading spinners for charms and bundles', function () {
    var jem = {
      listEnvironments: sinon.stub().callsArgWith(0, null, { envs: models })
    };
    charmstore.list = sinon.stub();
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      jem: jem,
      switchModel: sinon.stub(),
      showConnectingMask: sinon.stub(),
      interactiveLogin: true,
      changeState: sinon.stub(),
      storeUser: sinon.stub(),
      user: users.charmstore }), true);
    var output = component.getRenderOutput();
    assert.deepEqual(output.props.children.props.children.props.children[1].props.children[1], React.createElement(
      'div',
      { className: 'twelve-col' },
      React.createElement(juju.components.Spinner, null)
    ));
    assert.deepEqual(output.props.children.props.children.props.children[1].props.children[2], React.createElement(
      'div',
      { className: 'twelve-col' },
      React.createElement(juju.components.Spinner, null)
    ));
  });

  it('displays loading spinners for models', function () {
    var jem = {
      listEnvironments: sinon.stub()
    };
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      jem: jem,
      switchModel: sinon.stub(),
      showConnectingMask: sinon.stub(),
      interactiveLogin: true,
      changeState: sinon.stub(),
      storeUser: sinon.stub(),
      user: users.charmstore }), true);
    var output = component.getRenderOutput();
    assert.deepEqual(output.props.children.props.children.props.children[1].props.children[0], React.createElement(
      'div',
      { className: 'twelve-col' },
      React.createElement(juju.components.Spinner, null)
    ));
  });

  it('renders lists of entities', function () {
    var jem = {
      listEnvironments: sinon.stub().callsArgWith(0, null, { envs: models })
    };
    var changeState = sinon.stub();
    var getDiagramURL = sinon.stub();
    var switchModel = sinon.stub();
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: getDiagramURL,
      jem: jem,
      switchModel: switchModel,
      showConnectingMask: sinon.stub(),
      interactiveLogin: true,
      changeState: changeState,
      storeUser: sinon.stub(),
      user: users.charmstore }), true);
    var instance = component.getMountedInstance();
    var output = component.getRenderOutput();
    var expected = React.createElement(
      'div',
      { className: 'inner-wrapper' },
      React.createElement(juju.components.UserProfileHeader, {
        users: users,
        avatar: '',
        bundleCount: 1,
        charmCount: 1,
        environmentCount: 1,
        interactiveLogin: instance._interactiveLogin,
        username: users.charmstore.usernameDisplay }),
      React.createElement(
        'div',
        null,
        React.createElement(
          'div',
          null,
          React.createElement(
            'div',
            { className: 'user-profile__header twelve-col no-margin-bottom' },
            'Models',
            React.createElement(
              'span',
              { className: 'user-profile__size' },
              '(',
              1,
              ')'
            )
          ),
          React.createElement(
            'ul',
            { className: 'user-profile__list twelve-col' },
            React.createElement(
              'li',
              { className: 'user-profile__list-header twelve-col' },
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col' },
                'Name'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col four-col' },
                'Credential'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col' },
                'Last accessed'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col one-col' },
                'Units'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col last-col' },
                'Owner'
              )
            ),
            [React.createElement(
              juju.components.UserProfileEntity,
              {
                entity: models[0],
                expanded: false,
                key: 'env1',
                switchModel: instance.switchEnv,
                type: 'model' },
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col' },
                'sandbox'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col four-col' },
                '--'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col' },
                'today'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col one-col' },
                '--'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col last-col' },
                'test-owner'
              )
            )]
          )
        ),
        React.createElement(
          'div',
          null,
          React.createElement(
            'div',
            { className: 'user-profile__header twelve-col no-margin-bottom' },
            'Bundles',
            React.createElement(
              'span',
              { className: 'user-profile__size' },
              '(',
              1,
              ')'
            )
          ),
          React.createElement(
            'ul',
            { className: 'user-profile__list twelve-col' },
            React.createElement(
              'li',
              { className: 'user-profile__list-header twelve-col' },
              React.createElement(
                'span',
                { className: 'user-profile__list-col five-col' },
                'Name'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col user-profile__list-icons' },
                'Charms'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col one-col prepend-one' },
                'Units'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col last-col' },
                'Owner'
              )
            ),
            [React.createElement(
              juju.components.UserProfileEntity,
              {
                changeState: changeState,
                entity: bundles[0],
                getDiagramURL: getDiagramURL,
                key: 'django-cluster',
                type: 'bundle' },
              React.createElement(
                'span',
                { className: 'user-profile__list-col five-col ' + 'user-profile__list-name' },
                'django-cluster',
                React.createElement(
                  'ul',
                  { className: 'user-profile__list-tags' },
                  [React.createElement(
                    'li',
                    { className: 'user-profile__comma-item',
                      key: 'django-cluster-database' },
                    'database'
                  )]
                )
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col ' + 'user-profile__list-icons' },
                React.createElement('img', { className: 'user-profile__list-icon',
                  key: 'icon-0-gunicorn',
                  src: 'example.com/9/gunicorn/icon.svg',
                  title: 'gunicorn' }),
                React.createElement('img', { className: 'user-profile__list-icon',
                  key: 'icon-1-django',
                  src: 'example.com/9/django/icon.svg',
                  title: 'django' })
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col one-col prepend-one' },
                5
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col last-col' },
                'test-owner'
              )
            )]
          )
        ),
        React.createElement(
          'div',
          null,
          React.createElement(
            'div',
            { className: 'user-profile__header twelve-col no-margin-bottom' },
            'Charms',
            React.createElement(
              'span',
              { className: 'user-profile__size' },
              '(',
              1,
              ')'
            )
          ),
          React.createElement(
            'ul',
            { className: 'user-profile__list twelve-col' },
            React.createElement(
              'li',
              { className: 'user-profile__list-header twelve-col' },
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col' },
                'Name'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col seven-col' },
                'Series'
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col last-col' },
                'Owner'
              )
            ),
            [React.createElement(
              juju.components.UserProfileEntity,
              {
                changeState: changeState,
                entity: charms[0],
                key: 'cs:django',
                type: 'charm' },
              React.createElement(
                'span',
                { className: 'user-profile__list-col three-col ' + 'user-profile__list-name' },
                'django',
                React.createElement(
                  'ul',
                  { className: 'user-profile__list-tags' },
                  [React.createElement(
                    'li',
                    { className: 'user-profile__comma-item',
                      key: 'cs:django-database' },
                    'database'
                  )]
                )
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col four-col' },
                React.createElement(
                  'ul',
                  { className: 'user-profile__list-series' },
                  [React.createElement(
                    'li',
                    { className: 'user-profile__comma-item',
                      key: 'cs:django-trusty' },
                    'trusty'
                  )]
                )
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col one-col ' + 'user-profile__list-icons' },
                React.createElement('img', { className: 'user-profile__list-icon',
                  src: 'example.com/9/django/icon.svg',
                  title: 'django' })
              ),
              React.createElement(
                'span',
                { className: 'user-profile__list-col two-col ' + 'prepend-two last-col' },
                'test-owner'
              )
            )]
          )
        )
      )
    );
    assert.deepEqual(output.props.children.props.children, expected);
  });

  it('does not pass the charmstore login if interactiveLogin is falsy', function () {
    var output = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      charmstore: {},
      switchModel: sinon.stub(),
      listEnvs: sinon.stub(),
      showConnectingMask: sinon.stub(),
      changeState: sinon.stub(),
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      interactiveLogin: false,
      storeUser: sinon.stub(),
      user: users.charmstore }));
    var expected = React.createElement(juju.components.UserProfileHeader, {
      users: users,
      avatar: '',
      bundleCount: 0,
      charmCount: 0,
      environmentCount: 0,
      interactiveLogin: undefined,
      username: users.charmstore.usernameDisplay });
    assert.deepEqual(output.props.children.props.children.props.children[0], expected);
  });

  it('can log in to charmstore fetch macaroons from the bakery', function () {
    var macaroon = sinon.spy();
    var storeUser = sinon.stub();
    var charmstore = {
      bakery: {
        fetchMacaroonFromStaticPath: sinon.stub().callsArgWith(0, null, macaroon)
      }
    };
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      switchModel: sinon.stub(),
      users: users,
      listEnvs: sinon.stub(),
      showConnectingMask: sinon.stub(),
      changeState: sinon.stub(),
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      interactiveLogin: true,
      storeUser: storeUser,
      user: users.charmstore }), true);
    var instance = renderer.getMountedInstance();
    instance._interactiveLogin();
    assert.equal(charmstore.bakery.fetchMacaroonFromStaticPath.callCount, 1);
    assert.equal(storeUser.callCount, 1);
  });

  it('gets the entity data when the user authenticates', function () {
    var list = sinon.stub();
    var charmstore = { list: list };
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      switchModel: sinon.stub(),
      users: {},
      listEnvs: sinon.stub(),
      showConnectingMask: sinon.stub(),
      changeState: sinon.stub(),
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      interactiveLogin: true,
      storeUser: sinon.stub(),
      user: users.charmstore }), true);
    assert.equal(list.callCount, 0);
    component.render(React.createElement(juju.components.UserProfile, {
      switchModel: sinon.stub(),
      users: users,
      listEnvs: sinon.stub(),
      showConnectingMask: sinon.stub(),
      changeState: sinon.stub(),
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      interactiveLogin: true,
      storeUser: sinon.stub(),
      user: users.charmstore }));
    assert.equal(list.callCount, 2);
  });

  it('requests jem envs if jem is provided and updates state', function () {
    var jem = {
      listEnvironments: sinon.stub().callsArgWith(0, null, { envs: models })
    };
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      switchModel: sinon.stub(),
      users: users,
      changeState: sinon.stub(),
      charmstore: {},
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      showConnectingMask: sinon.stub(),
      storeUser: sinon.stub(),
      user: users.charmstore,
      jem: jem }), true);
    var instance = component.getMountedInstance();

    assert.equal(jem.listEnvironments.callCount, 1);
    assert.deepEqual(instance.state.envList, models);
  });

  it('requests jes envs if no jem is provided and updates state', function () {
    var listEnvs = sinon.stub().callsArgWith(1, { envs: models });
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      switchModel: sinon.stub(),
      users: users,
      changeState: sinon.stub(),
      charmstore: {},
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      showConnectingMask: sinon.stub(),
      storeUser: sinon.stub(),
      user: users.charmstore,
      listEnvs: listEnvs }), true);
    var instance = component.getMountedInstance();

    assert.equal(listEnvs.args[0][0], 'user-admin');
    assert.deepEqual(instance.state.envList, models);
  });

  it('switches env when calling switchModel method passed to list', function () {
    // This method is passed down to child components and called from there.
    // We are just calling it directly here to unit test the method.
    var switchModel = sinon.stub();
    var changeState = sinon.stub();
    var listEnvs = sinon.stub();
    var showMask = sinon.stub();
    var dbset = sinon.stub();
    var envs = [{
      uuid: 'abc123',
      user: 'foo',
      password: 'bar'
    }];
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      switchModel: switchModel,
      users: users,
      changeState: changeState,
      charmstore: {},
      env: env,
      getDiagramURL: sinon.stub(),
      showConnectingMask: showMask,
      storeUser: sinon.stub(),
      user: users.charmstore,
      dbEnvironmentSet: dbset,
      listEnvs: listEnvs }), true);
    var instance = component.getMountedInstance();
    // Call the callback for the listEnvs call to populate the state.
    listEnvs.args[0][1](envs);
    // Call the method that's passed down. We test that this method is correctly
    // passed down in the initial 'happy path' full rendering test.
    instance.switchEnv('abc123', 'modelname');
    // Make sure we show the canvas loading mask when switching models.
    assert.equal(showMask.callCount, 1);
    // We need to call to generate the proper socket URL.
    // Check that switchModel is called with the proper values.
    assert.equal(switchModel.callCount, 1, 'switchEnv not called');
    assert.deepEqual(switchModel.args[0], ['abc123', [{
      uuid: 'abc123', user: 'foo', password: 'bar' }]]);
    // The database needs to be updated with the new model name.
    assert.equal(dbset.callCount, 1);
    assert.deepEqual(dbset.args[0], ['name', 'modelname']);
    // Make sure we close the profile page when switching envs.
    assert.equal(changeState.callCount, 1, 'changeState not called');
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: null,
        metadata: null
      }
    });
  });

  it('requests entities and updates state', function () {
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      changeState: sinon.stub(),
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      interactiveLogin: true,
      listEnvs: sinon.stub(),
      showConnectingMask: sinon.stub(),
      storeUser: sinon.stub(),
      switchModel: sinon.stub(),
      user: users.charmstore }), true);
    var instance = component.getMountedInstance();
    assert.equal(charmstore.list.callCount, 2, 'charmstore list not called');
    assert.equal(charmstore.list.args[0][0], 'test-owner', 'username not passed to list request');
    assert.deepEqual(instance.state.charmList, charms, 'callback does not properly set charm state');
    assert.deepEqual(instance.state.bundleList, bundles, 'callback does not properly set bundle state');
  });

  it('gets the default model for older versions of Juju', function () {
    env.findFacadeVersion = sinon.stub().returns(null);
    var component = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      changeState: sinon.stub(),
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      interactiveLogin: true,
      listEnvs: sinon.stub(),
      showConnectingMask: sinon.stub(),
      storeUser: sinon.stub(),
      switchModel: sinon.stub(),
      user: users.charmstore }), true);
    var instance = component.getMountedInstance();
    component.getRenderOutput();
    assert.deepEqual(instance.state.envList, [{
      name: 'default',
      owner: 'test owner',
      uuid: '',
      lastConnection: 'now'
    }]);
  });

  it('will abort the requests when unmounting', function () {
    var charmstoreAbort = sinon.stub();
    var listEnvsAbort = sinon.stub();
    var listEnvs = sinon.stub().returns({ abort: listEnvsAbort });
    charmstore.list = sinon.stub().returns({ abort: charmstoreAbort });
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.UserProfile, {
      users: users,
      changeState: sinon.stub(),
      charmstore: charmstore,
      dbEnvironmentSet: sinon.stub(),
      env: env,
      getDiagramURL: sinon.stub(),
      interactiveLogin: true,
      listEnvs: listEnvs,
      showConnectingMask: sinon.stub(),
      storeUser: sinon.stub(),
      switchModel: sinon.stub(),
      username: users.charmstore.user }), true);
    renderer.unmount();
    assert.equal(charmstoreAbort.callCount, 2);
    assert.equal(listEnvsAbort.callCount, 1);
  });
});