/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('user-profile', function () {

  juju.components.UserProfile = React.createClass({
    xhrs: [],

    propTypes: {
      changeState: React.PropTypes.func.isRequired,
      charmstore: React.PropTypes.object.isRequired,
      currentModel: React.PropTypes.string,
      dbEnvironmentSet: React.PropTypes.func.isRequired,
      env: React.PropTypes.object.isRequired,
      getDiagramURL: React.PropTypes.func.isRequired,
      interactiveLogin: React.PropTypes.bool,
      jem: React.PropTypes.object,
      listEnvs: React.PropTypes.func,
      showConnectingMask: React.PropTypes.func.isRequired,
      storeUser: React.PropTypes.func.isRequired,
      switchModel: React.PropTypes.func.isRequired,
      users: React.PropTypes.object.isRequired,
      user: React.PropTypes.object
    },

    getInitialState: function getInitialState() {
      return {
        envList: [],
        charmList: [],
        bundleList: [],
        loadingBundles: false,
        loadingCharms: false,
        loadingModels: false
      };
    },

    componentWillMount: function componentWillMount() {
      var props = this.props,
          users = props.users;
      this._fetchEnvironments(props);
      if (users.charmstore && users.charmstore.user) {
        this._fetchEntities('charm', props);
        this._fetchEntities('bundle', props);
      }
    },

    componentWillUnmount: function componentWillUnmount() {
      this.xhrs.forEach(function (xhr) {
        xhr && xhr.abort && xhr.abort();
      });
    },

    componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
      // If the user has changed then update the data.
      var props = this.props;
      if (nextProps.user.user !== props.user.user) {
        this._fetchEnvironments(nextProps);
      }
      // Compare next and previous charmstore users in a data-safe manner.
      var prevCSUser = props.users.charmstore || {};
      var nextCSUser = nextProps.users.charmstore || {};
      if (nextCSUser.user !== prevCSUser.user) {
        this._fetchEntities('charm', nextProps);
        this._fetchEntities('bundle', nextProps);
      }
    },

    /**
      Makes a request of JEM or JES to fetch the users availble environments.
       @method _fetchEnvironments
      @param {Object} props the properties to use when connection to JEM/JES
    */
    _fetchEnvironments: function _fetchEnvironments(props) {
      var _this = this;

      var jem = props.jem;
      this.setState({ loadingModels: true }, function () {
        // Delay the call until after the state change to prevent race
        // conditions.
        var env = _this.props.env;
        if (env.findFacadeVersion('ModelManager') === null && env.findFacadeVersion('EnvironmentManager') === null) {
          // If we're on Juju < 2 then pass the default model to the list.
          var environmentName = env.get('environmentName');
          var username = _this.props.user && _this.props.user.usernameDisplay;
          _this._fetchEnvironmentsCallback(null, {
            environmentCount: 1,
            envs: [{
              name: environmentName,
              owner: username,
              // Leave the UUID blank so that it navigates to the default model
              // when selected.
              uuid: '',
              lastConnection: 'now'
            }]
          });
          return;
        }
        var xhr;
        if (jem) {
          xhr = jem.listEnvironments(_this._fetchEnvironmentsCallback);
        } else {
          xhr = props.listEnvs('user-admin', _this._fetchEnvironmentsCallback.bind(_this, null));
        }
        _this.xhrs.push(xhr);
      });
    },

    /**
      Callback for the JEM and JES list environments call.
       @method _fetchEnvironmentsCallback
      @param {String} error The error from the request, or null.
      @param {Object} data The data from the request.
    */
    _fetchEnvironmentsCallback: function _fetchEnvironmentsCallback(error, data) {
      this.setState({ loadingModels: false });
      // We need to coerce error types returned by JES vs JEM into one error.
      var err = data.err || error;
      if (err) {
        console.log(err);
        return;
      }
      // data.envs is only populated in the JES environments, when using JEM
      // the environments are in the top level 'data' object.
      this.setState({ envList: data.envs || data });
    },

    /**
      Requests a list from charmstore of the user's entities.
       @method _fetchEntities
      @param {String} type the entity type, charm or bundle
      @param {Object} props the properties to use when connection to charmstore
    */
    _fetchEntities: function _fetchEntities(type, props) {
      var _this2 = this;

      var callback = this._fetchEntitiesCallback.bind(this, type);
      var charmstore = props.charmstore;
      var username = props.users.charmstore && props.users.charmstore.user;
      if (charmstore && charmstore.list && username) {
        var state = {};
        if (type === 'charm') {
          state.loadingCharms = true;
        } else {
          state.loadingBundles = true;
        }
        this.setState(state, function () {
          // Delay the call until after the state change to prevent race
          // conditions.
          var xhr = charmstore.list(username, callback, type);
          _this2.xhrs.push(xhr);
        });
      }
    },

    /**
      Callback for the request to list a user's entities.
       @method _fetchEntitiesCallback
      @param {String} error The error from the request, or null.
      @param {Object} data The data from the request.
    */
    _fetchEntitiesCallback: function _fetchEntitiesCallback(type, error, data) {
      // Turn off the loader, regardless of success or error.
      if (type === 'charm') {
        this.setState({ loadingCharms: false });
      } else if (type === 'bundle') {
        this.setState({ loadingBundles: false });
      }
      if (error) {
        console.log(error);
        return;
      }
      // Pull out just the data we need to display.
      if (type === 'charm') {
        this.setState({ charmList: data });
      } else if (type === 'bundle') {
        this.setState({ bundleList: data });
      }
    },

    /**
      Take the supplied UUID, fetch the username and password then call the
      passed in switchModel method.
       @method switchEnv
      @param {String} uuid The env UUID.
      @param {String} name The env name.
    */
    switchEnv: function switchEnv(uuid, name) {
      var props = this.props;
      props.showConnectingMask();
      props.switchModel(uuid, this.state.envList);
      props.dbEnvironmentSet('name', name);
      this.close();
    },

    close: function close() {
      this.props.changeState({
        sectionC: {
          component: null,
          metadata: null
        }
      });
    },

    /**
      Calls the bakery to get a charmstore macaroon.
       @method _interactiveLogin
    */
    _interactiveLogin: function _interactiveLogin() {
      var bakery = this.props.charmstore.bakery;
      bakery.fetchMacaroonFromStaticPath(this._fetchMacaroonCallback);
    },

    /**
      Callback for fetching the macaroon.
       @method _fetchMacaroonCallback
      @param {String|Object|Null} error The error response from the callback.
      @param {String} macaroon The resolved macaroon.
    */
    _fetchMacaroonCallback: function _fetchMacaroonCallback(error, macaroon) {
      if (error) {
        console.log(error);
      } else {
        this.props.storeUser('charmstore', true);
      }
    },

    /**
      Generate the details for the provided model.
       @method _generateModelRow
      @param {Object} model A model object.
      @returns {Array} The markup for the row.
    */
    _generateModelRow: function _generateModelRow(model) {
      var uuid = model.uuid;
      return React.createElement(
        juju.components.UserProfileEntity,
        {
          entity: model,
          expanded: model.name === this.props.currentModel,
          key: uuid,
          switchModel: this.switchEnv,
          type: 'model' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col' },
          model.name
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col four-col' },
          '--'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col' },
          model.lastConnection
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col' },
          '--'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          model.owner
        )
      );
    },

    /**
      Generate the header for the models.
       @method _generateModelHeader
      @returns {Array} The markup for the header.
    */
    _generateModelHeader: function _generateModelHeader() {
      return React.createElement(
        'li',
        { className: 'user-profile__list-header twelve-col' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col' },
          'Name'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col four-col' },
          'Credential'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col' },
          'Last accessed'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col' },
          'Units'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          'Owner'
        )
      );
    },

    /**
      Generate a list of tags.
       @method _generateTags
      @param {Array} tags A list of tags.
      @param {String} id The id of the entity.
      @returns {Object} A list of tag components.
    */
    _generateTags: function _generateTags(tagList, id) {
      if (!tagList) {
        return;
      }
      var tags = [];
      tagList.forEach(function (tag) {
        tags.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item',
            key: id + '-' + tag },
          tag
        ));
      });
      return React.createElement(
        'ul',
        { className: 'user-profile__list-tags' },
        tags
      );
    },

    /**
      Generate a list of series.
       @method _generateSeries
      @param {Array} series A list of series.
      @param {String} id The id of the entity.
      @returns {Object} A list of series components.
    */
    _generateSeries: function _generateSeries(series, id) {
      if (!series) {
        return;
      }
      var listItems = [];
      series.forEach(function (release) {
        listItems.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item',
            key: id + '-' + release },
          release
        ));
      });
      return React.createElement(
        'ul',
        { className: 'user-profile__list-series' },
        listItems
      );
    },
    /**
      Construct the URL for a service icon.
       @method _getIcon
      @param {String} id The service ID.
      @returns {String} The icon URL.
    */
    _getIcon: function _getIcon(id) {
      if (!id) {
        return;
      }
      var cs = this.props.charmstore;
      var path = id.replace('cs:', '');
      return '' + cs.url + cs.version + '/' + path + '/icon.svg';
    },

    /**
      Generate the details for the provided bundle.
       @method _generateBundleRow
      @param {Object} bundle A bundle object.
      @returns {Array} The markup for the row.
    */
    _generateBundleRow: function _generateBundleRow(bundle) {
      var _this3 = this;

      var id = bundle.id;
      var services = [];
      var serviceNames = Object.keys(bundle.services);
      serviceNames.forEach(function (serviceName, idx) {
        var service = bundle.services[serviceName];
        var id = service.charm;
        var key = 'icon-' + idx + '-' + id;
        services.push(React.createElement('img', { className: 'user-profile__list-icon',
          key: key,
          src: _this3._getIcon(id),
          title: service.charm }));
      });
      var unitCount = bundle.unitCount || React.createElement(
        'span',
        null,
        ' '
      );
      return React.createElement(
        juju.components.UserProfileEntity,
        {
          changeState: this.props.changeState,
          entity: bundle,
          getDiagramURL: this.props.getDiagramURL,
          key: id,
          type: 'bundle' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col five-col ' + 'user-profile__list-name' },
          bundle.name,
          this._generateTags(bundle.tags, id)
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col ' + 'user-profile__list-icons' },
          services
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col prepend-one' },
          unitCount
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          bundle.owner
        )
      );
    },

    /**
      Generate the header for the bundles.
       @method _generateBundleHeader
      @returns {Array} The markup for the header.
    */
    _generateBundleHeader: function _generateBundleHeader() {
      return React.createElement(
        'li',
        { className: 'user-profile__list-header twelve-col' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col five-col' },
          'Name'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col ' + 'user-profile__list-icons' },
          'Charms'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col prepend-one' },
          'Units'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          'Owner'
        )
      );
    },

    /**
      Generate the details for the provided charm.
       @method _generateCharmRow
      @param {Object} charm A charm object.
      @returns {Array} The markup for the row.
    */
    _generateCharmRow: function _generateCharmRow(charm) {
      var id = charm.id;
      // Ensure the icon is set.
      charm.icon = charm.icon || this._getIcon(id);
      return React.createElement(
        juju.components.UserProfileEntity,
        {
          changeState: this.props.changeState,
          entity: charm,
          key: id,
          type: 'charm' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col ' + 'user-profile__list-name' },
          charm.name,
          this._generateTags(charm.tags, id)
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col four-col' },
          this._generateSeries(charm.series, id)
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col one-col ' + 'user-profile__list-icons' },
          React.createElement('img', { className: 'user-profile__list-icon',
            src: charm.icon,
            title: charm.name })
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col ' + 'prepend-two last-col' },
          charm.owner
        )
      );
    },

    /**
      Generate the header for the charms.
       @method _generateCharmHeader
      @returns {Array} The markup for the header.
    */
    _generateCharmHeader: function _generateCharmHeader() {
      return React.createElement(
        'li',
        { className: 'user-profile__list-header twelve-col' },
        React.createElement(
          'span',
          { className: 'user-profile__list-col three-col' },
          'Name'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col seven-col' },
          'Series'
        ),
        React.createElement(
          'span',
          { className: 'user-profile__list-col two-col last-col' },
          'Owner'
        )
      );
    },

    /**
      Generate the rows of for the provided entity.
       @method _generateRows
      @param {String} type The type of entity.
      @param {Array} list The list of entities.
      @param {Boolean} loading Whether the data is loading.
      @returns {Array} The markup for the rows.
    */
    _generateRows: function _generateRows(type, list, loading) {
      if (loading) {
        return React.createElement(
          'div',
          { className: 'twelve-col' },
          React.createElement(juju.components.Spinner, null)
        );
      }
      if (list.length === 0) {
        return;
      }
      var generateRow;
      var header;
      var rows = [];
      var title;
      if (type === 'models') {
        generateRow = this._generateModelRow;
        header = this._generateModelHeader();
        title = 'Models';
      } else if (type === 'bundles') {
        generateRow = this._generateBundleRow;
        header = this._generateBundleHeader();
        title = 'Bundles';
      } else if (type === 'charms') {
        generateRow = this._generateCharmRow;
        header = this._generateCharmHeader();
        title = 'Charms';
      }
      list.forEach(function (model) {
        rows.push(generateRow(model));
      });
      return React.createElement(
        'div',
        null,
        React.createElement(
          'div',
          { className: 'user-profile__header twelve-col no-margin-bottom' },
          title,
          React.createElement(
            'span',
            { className: 'user-profile__size' },
            '(',
            list.length,
            ')'
          )
        ),
        React.createElement(
          'ul',
          { className: 'user-profile__list twelve-col' },
          header,
          rows
        )
      );
    },

    /**
      Generate the content for the panel.
       @method _generateContent
      @returns {Array} The markup for the content.
    */
    _generateContent: function _generateContent() {
      var state = this.state;
      // We can't be loading anything, and all the lists must be empty.
      var isEmpty = !state.loadingBundles && !state.loadingCharms && !state.loadingModels && state.bundleList.length === 0 && state.charmList.length === 0 && state.envList.length === 0;
      if (isEmpty) {
        return React.createElement(
          'div',
          { className: 'user-profile__empty twelve-col no-margin-bottom' },
          React.createElement('img', { alt: 'Empty profile',
            className: 'user-profile__empty-image',
            src: '/juju-ui/assets/images/non-sprites/empty_profile.png' }),
          React.createElement(
            'h2',
            { className: 'user-profile__empty-title' },
            'Your profile is currently empty'
          ),
          React.createElement(
            'p',
            { className: 'user-profile__empty-text' },
            'Your models, bundles and charms will appear here when you create them.'
          )
        );
      }
      return React.createElement(
        'div',
        null,
        this._generateRows('models', state.envList, state.loadingModels),
        this._generateRows('bundles', state.bundleList, state.loadingBundles),
        this._generateRows('charms', state.charmList, state.loadingCharms)
      );
    },

    render: function render() {
      var username = this.props.user && this.props.user.usernameDisplay;
      return React.createElement(
        juju.components.Panel,
        {
          instanceName: 'user-profile',
          visible: true },
        React.createElement(
          'div',
          { className: 'twelve-col' },
          React.createElement(
            'div',
            { className: 'inner-wrapper' },
            React.createElement(juju.components.UserProfileHeader, {
              users: this.props.users,
              avatar: '',
              bundleCount: this.state.bundleList.length,
              charmCount: this.state.charmList.length,
              environmentCount: this.state.envList.length,
              interactiveLogin: this.props.interactiveLogin ? this._interactiveLogin : undefined,
              username: username }),
            this._generateContent()
          )
        )
      );
    }

  });
}, '', {
  requires: ['loading-spinner', 'svg-icon', 'panel-component', 'user-profile-entity', 'user-profile-header']
});